# describe.py: Explain details to encode for Julython.
# Copyright (C) 2013  Stephen Compall

# Julython-commit is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or (at
# your option) any later version.

# Julython-commit is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# General Public License for more details.

# You should have received a copy of the GNU General Public License
# along with Julython-commit.  If not, see <http://www.gnu.org/licenses/>.

from operator import attrgetter
import urllib as url

from zope.interface import Interface, implementer
from zope.schema import URI, Datetime, Text, TextLine

from julythoncommit.schemautil import zs

def commit_pattern_p(some_string):
    return True                 # TODO

class IConfig(Interface):
    "How to map a set of commits to a Julython message."
    repo_url = URI(title=u'Repository URL')
    repo_name = TextLine(title=u'Repository name')
    repo_description = Text(title=u'Description')
    commit_pattern = TextLine(title=u'Pattern mapping revids to their URLs',
                              constraint=commit_pattern_p)

class IRevision(Interface):
    "Information about a single revision."
    email = TextLine(title=u"Author email")
    author_name = TextLine(title=u"Author name")
    revision_id = Text(title=u"Revision ID")
    message = TextLine(title=u"Message")
    timestamp = Datetime(title=u"Time of commit")

@implementer(IConfig)
class Config(object):
    def __init__(self, repo_url, repo_name, repo_description, commit_pattern):
        self.repo_url = repo_url
        self.repo_name = repo_name
        self.repo_description = repo_description
        self.commit_pattern = commit_pattern

    def commit_url(self, revid):
        """Guess the URL for a revid."""
        return self.commit_pattern % url.quote(revid, '')

    def repository_json(self):
        """Produce enough bits for Julython's GithubHandler
        [repository]."""
        return {
            'url': unicode(self.repo_url),
            'description': self.repo_description,
            'name': self.repo_name,
            'forks': 0,
            'watchers': 0,
        }

    def revision_json(self, rev):
        """Produce enough bits for Julython's GithubHandler
        [commit]."""
        zs(IRevision, rev)
        return {
            'author': {
                'email': rev.email,
                'name': rev.author_name,
            },
            'id': rev.revision_id,
            'url': unicode(self.commit_url(rev.revision_id)),
            'message': rev.message,
            'timestamp': rev.timestamp.isoformat(),
        }

    def push_json(self, basis_id, revs):
        """Produce all the bits for a GithubHandler push."""
        revs = sorted(revs, key=attrgetter('timestamp'))
        return {
            'before': basis_id,
            'after': revs[-1].revision_id if revs else basis_id,
            'ref': 'refs/heads/master', # XXX fake, not sure it matters
            'repository': self.repository_json(),
            'commits': [self.revision_json(rev) for rev in revs],
        }

@implementer(IRevision)
class Revision(object):
    def __init__(self, email, author_name, revision_id, message, timestamp):
        self.email = email
        self.author_name = author_name
        self.revision_id = revision_id
        self.message = message
        self.timestamp = timestamp

# describe.py ends here
